import Vue from "vue";
import VueRouter from "@/vue-router";
import AleksisVue from "./plugins/aleksis.js";

import Vuetify from "@/vuetify";
import "@/@mdi/font/css/materialdesignicons.css";
import "@/vuetify/dist/vuetify.min.css";

import { ApolloClient, HttpLink, from } from "@/apollo-boost";
import VueApollo from "@/vue-apollo";
import { InMemoryCache } from "@/apollo-cache-inmemory";
import { onError } from "@/apollo-link-error";
import { RetryLink } from "@/apollo-link-retry";
import { persistCache, LocalStorageWrapper } from "@/apollo3-cache-persist";

import gql from "@/graphql-tag";

import gqlSnackbarItems from "./snackbarItems.graphql";

import "./css/global.scss";
import VueI18n from "@/vue-i18n";

import dateTimeFormats from "./dateTimeFormats.js";
import routes from "./routes.js";

Vue.use(Vuetify);

const vuetify = new Vuetify({
  icons: {
    iconfont: "mdi", // default - only for display purposes
    values: {
      cancel: "mdi-close-circle-outline",
      delete: "mdi-close-circle-outline",
      success: "mdi-check-circle-outline",
      info: "mdi-information-outline",
      warning: "mdi-alert-outline",
      error: "mdi-alert-octagon-outline",
      prev: "mdi-chevron-left",
      next: "mdi-chevron-right",
      checkboxOn: "mdi-checkbox-marked-outline",
      checkboxIndeterminate: "mdi-minus-box-outline",
      edit: "mdi-pencil-outline",
      preferences: "mdi-cog-outline",
    },
  },
});

Vue.use(VueI18n);

const i18n = new VueI18n({
  locale: "en",
  fallbackLocale: "en",
  messages: {},
  dateTimeFormats,
});

// Using this function, apps can register their locale files
i18n.registerLocale = function (messages) {
  for (let locale in messages) {
    i18n.mergeLocaleMessage(locale, messages[locale]);
  }
};

Vue.use(VueApollo);
Vue.use(VueRouter);
Vue.use(AleksisVue);

export const typeDefs = gql`
  type snackbarItem {
    id: ID!
    messageKey: String!
    color: String!
    read: Boolean!
  }

  type globalState {
    contentLoading: Boolean!
    browserTitle: String
  }

  type Mutation {
    checkSnackbarItem(id: ID!): Boolean
    setLoading(state: Boolean!): Boolean
  }
`;

const cache = new InMemoryCache();

const resolvers = {
  Mutation: {
    // eslint-disable-next-line no-unused-vars
    checkSnackbarItem: (_, { id }, { cache }) => {
      const data = cache.readQuery({ query: gqlSnackbarItems });
      const currentItem = data.snackbarItems.find((item) => item.id === id);
      currentItem.read = !currentItem.read;
      cache.writeQuery({ query: gqlSnackbarItems, data });
      return currentItem.read;
    },
    // eslint-disable-next-line
    setLoading: (_, { state }, {}) => {
      return true;
    },
  },
};

function addErrorSnackbarItem(messageKey) {
  let uuid = crypto.randomUUID();
  cache.writeQuery({
    query: gqlSnackbarItems,
    data: {
      snackbarItems: [
        {
          __typename: "snackbarItem",
          id: uuid,
          messageKey: messageKey,
          color: "red",
          read: false,
        },
      ],
    },
    variables: {
      id: uuid,
    },
  });
}

const retryLink = new RetryLink();

const errorLink = onError(({ graphQLErrors, networkError }) => {
  if (graphQLErrors) addErrorSnackbarItem("graphql.snackbar_error_message");

  if (networkError)
    addErrorSnackbarItem("network_errors.snackbar_error_message");
});

const httpLink = new HttpLink({
  uri: window.location.origin + "/graphql/",
});

// FIXME: has to be async to guarantee that no entries are cached before the cache is persisted.
persistCache({
  cache: cache,
  storage: new LocalStorageWrapper(window.localStorage),
});

const apolloClient = new ApolloClient({
  typeDefs,
  resolvers,
  cache,
  link: from([retryLink, errorLink, httpLink]),
});

import App from "./App.vue";
import MessageBox from "./components/MessageBox.vue";

Vue.component(MessageBox.name, MessageBox); // Load MessageBox globally as other components depend on it

Vue.use(VueApollo);

const apolloProvider = new VueApollo({
  defaultClient: apolloClient,
});

const router = new VueRouter({
  mode: "history",
  routes,
});

router.afterEach((to, from, next) => {
  app.$setPageTitle(null, to);
});
if (document.getElementById("sentry_settings") !== null) {
  const Sentry = import("@sentry/vue");
  const { BrowserTracing } = import("@sentry/tracing");

  const sentry_settings = JSON.parse(
    document.getElementById("sentry_settings").textContent
  );
  Sentry.init({
    Vue,
    dsn: sentry_settings.dsn,
    environment: sentry_settings.environment,
    tracesSampleRate: sentry_settings.traces_sample_rate,
    logError: true,
    integrations: [
      new BrowserTracing({
        routingInstrumentation: Sentry.vueRouterInstrumentation(router),
      }),
    ],
  });
}

const app = new Vue({
  el: "#app",
  apolloProvider,
  vuetify: vuetify,
  render: (h) => h(App),
  data: () => ({
    showCacheAlert: false,
    contentLoading: false,
    pageBaseTitle: document.title,
  }),
  router,
  i18n,
});

// eslint-disable-next-line no-unused-vars
router.beforeEach((to, from, next) => {
  app.contentLoading = true;
  next();
});

// eslint-disable-next-line no-unused-vars
router.afterEach((to, from) => {
  Vue.nextTick(() => {
    app.contentLoading = false;
  });
});

// eslint-disable-next-line no-unused-vars
router.beforeEach((to, from, next) => {
  console.debug("Setting new page title due to route change");
  app.$setPageTitle(null, to);
  next();
});

window.app = app;
window.router = router;
window.i18n = i18n;

import("./messages.js");
